package # Hide from PAUSE
  DBIx::Class::Version::Table;

package # Hide from PAUSE
  Dancer::Plugin::Interchange6::Cart::DBIC;

=head1 DESCRIPTION

Dummy module to cleanup old installs

=cut

# dummy module to cleanup old installs
1;
